/*
 * $Id: BulletImpl.as,v 1.1.1.1 2006/07/17 03:27:32 kenta Exp $
 *
 * Copyright 2006 Kenta Cho. Some rights reserved.
 */
package abagames.dngn {
  import flash.display.BitmapData;
  import flash.geom.Rectangle;
  import abagames.util.bulletml.BulletML;
  import abagames.util.bulletml.BulletPool;
  import abagames.util.bulletml.Bullet;

  /**
   * Bullet.
   */
  public class BulletImpl implements Bullet {
    public static var screenWidth:int;
    public static var screenHeight:int;
    public static var scale:Number = 1;
    public static var rank:Number = 0.5;
    public static var color:uint = 0xffffff;
    public static var rect:Rectangle = new Rectangle(0, 0, 5, 5);
    private static var rCnt:Number = 0;
    private static var gCnt:Number = 0;
    private static var bCnt:Number = 0;
    private static var dotColor:uint = 0xffffff;
    private var bml:BulletML;
    private var bp:BulletPool;
    private var x:Number;
    private var y:Number;
    private var rotation:Number;
    private var speed:Number;
    private var acx:Number;
    private var acy:Number;
    private var isVanished:Boolean;

    public function BulletImpl(bml:BulletML, bp:BulletPool,
                               x:Number, y:Number, deg:Number, speed:Number) {
      this.bml = bml;
      this.bp = bp;
      this.x = x;
      this.y = y;
      rotation = deg;
      this.speed = speed;
      acx = acy = 0;
      isVanished = false;
    }

    public static function updateDotColor():void {
      var r:uint = Math.floor(Math.abs(Math.cos(rCnt)) * 255) << 16;
      var g:uint = Math.floor(Math.abs(Math.cos(gCnt)) * 255) << 8;
      var b:uint = Math.floor(Math.abs(Math.cos(bCnt)) * 255);
      dotColor = r | g | b;
      rCnt += 0.011;
      gCnt += 0.007;
      bCnt += 0.009;
    }

    public function run(buf:BitmapData, dots:Dots = null):void {
      bml.run(this, bp);
      x += Math.sin(rotation * Math.PI / 180) * speed * scale;
      y -= Math.cos(rotation * Math.PI / 180) * speed * scale;
      x += acx * scale;
      y += acy * scale;
      if (x < 0 || x >= screenWidth || y < 0 || y >= screenHeight) {
        vanish();
        return;
      }
      rect.x = x - rect.width / 2;
      rect.y = y - rect.height / 2;
      buf.fillRect(rect, color);
      if (dots != null)
	dots.pset(rect.x, rect.y, dotColor);
    }

    public function isEnd():Boolean {
      return isVanished;
    }

    public function getDirection():Number {
      return rotation;
    }

    public function setDirection(v:Number):void {
      rotation = v;
    }

    public function getAimDirection():Number {
      return 180;
    }

    public function getSpeed():Number {
      return speed;
    }

    public function setSpeed(v:Number):void {
      speed = v;
    }

    public function getX():Number {
      return x;
    }

    public function getY():Number {
      return y;
    }

    public function setPos(x:Number, y:Number):void {
      this.x = x;
      this.y = y;
    }

    public function getHorizontal():Number {
      return acx;
    }

    public function setHorizontal(v:Number):void {
      acx = v;
    }

    public function getVertical():Number {
      return acy;
    }

    public function setVertical(v:Number):void {
      acy = v;
    }

    public function vanish():void {
      isVanished = true;
    }

    public function getRand():Number {
      return Math.random();
    }

    public function getRank():Number {
      return rank;
    }
  }
}
